// ---------------------------------------------------------------------------
// - Libaodc.cpp                                                             -
// - afnix:odc library - declaration & implementation                        -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2005 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "Meta.hpp"
#include "Libaodc.hpp"
#include "Predaodc.hpp"
#include "Function.hpp"
#include "Odccalls.hpp"
#include "Collection.hpp"

namespace afnix {
  // initialize the the library

  Object* init_afnix_odc (Interp* interp, Vector* argv) {
    // make sure we are not called from something crazy
    if (interp == nilp) return nilp;
    
    // create the afnix:odc nameset
    Nameset* aset = interp->mknset ("afnix", interp->getgset ());
    Nameset* aodc = interp->mknset ("odc",   aset);
 
    // bind all classes in the afnix:net nameset
    aodc->symcst ("Cell",         new Meta (Cell::mknew));
    aodc->symcst ("Table",        new Meta (Table::mknew));
    aodc->symcst ("Record",       new Meta (Record::mknew));
    aodc->symcst ("Collection",   new Meta (Collection::mknew));

    // bind all predicates in the afnix:odc nameset
    aodc->symcst ("cell-p",       new Function (odc_celp));
    aodc->symcst ("table-p",      new Function (odc_tblp));
    aodc->symcst ("record-p",     new Function (odc_rcdp));
    aodc->symcst ("collection-p", new Function (odc_colp));
       
    // bind the standard functions
    aodc->symcst ("read",         new Function (odc_read));

    // not used but needed
    return nilp;
  }
}

extern "C" {
  afnix::Object* dli_afnix_odc (afnix::Interp* interp, afnix::Vector* argv) {
    return init_afnix_odc (interp, argv);
  }
}
