// ---------------------------------------------------------------------------
// - Libatxt.cpp                                                             -
// - afnix:txt library - declaration & implementation                        -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2005 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "Meta.hpp"
#include "Sort.hpp"
#include "Hasher.hpp"
#include "Scanner.hpp"
#include "Libatxt.hpp"
#include "Literate.hpp"
#include "Predatxt.hpp"
#include "Function.hpp"

namespace afnix {
  Object* init_afnix_txt (Interp* interp, Vector* argv) {
    // make sure we are not called from something crazy
    if (interp == nilp) return nilp;
    
    // create the afnix:sys nameset
    Nameset* aset = interp->mknset ("afnix", interp->getgset ());
    Nameset* atxt = interp->mknset ("txt",   aset);
    
    // bind all classes in the afnix:txt nameset 
    atxt->symcst ("Hasher",       new Meta (Hasher::meval, Hasher::mknew));
    atxt->symcst ("Lexeme",       new Meta (Lexeme::mknew));
    atxt->symcst ("Pattern",      new Meta (Pattern::meval, Pattern::mknew));
    atxt->symcst ("Scanner",      new Meta (Scanner::mknew));
    atxt->symcst ("Literate",     new Meta (Literate::mknew));
   
    // bind the special functions
    atxt->symcst ("sort-ascent",  new Function (atxt_qsort_lth));
    atxt->symcst ("sort-descent", new Function (atxt_qsort_gth));
    atxt->symcst ("sort-lexical", new Function (atxt_qsort_lex));

    // bind all predicates in the afnix:txt nameset
    atxt->symcst ("hasher-p",     new Function (atxt_hashp));
    atxt->symcst ("lexeme-p",     new Function (atxt_lexp));
    atxt->symcst ("pattern-p",    new Function (atxt_patp));
    atxt->symcst ("scanner-p",    new Function (atxt_scanp));
    atxt->symcst ("literate-p",   new Function (atxt_tlitp));
    
    // not used but needed
    return nilp;
  }
}

extern "C" {
  afnix::Object* dli_afnix_txt (afnix::Interp* interp, afnix::Vector* argv) {
    return init_afnix_txt (interp, argv);
  }
}
