// ---------------------------------------------------------------------------
// - Digest.cpp                                                              -
// - afnix cryptography - base message digest class implementation           -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2005 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "Digest.hpp"

namespace afnix {

  // create a digets by name and size

  Digest::Digest (const String& name, const long size) {
    d_name = name;
    d_size = size;
    p_data = new t_byte[d_size];
    reset ();
  }

  // destroy this buffer
  
  Digest::~Digest (void) {
    delete [] p_data;
  }

  // reset this digest buffer

  void Digest::reset (void) {
    d_count  = 0;
    d_length = 0;
  }

  // clear the digest buffer counter

  void Digest::clear (void) {
    d_count = 0;
  }

  // return the digest name

  String Digest::getname (void) const {
    return d_name;
  }

  // get the digest buffer count
  
  long Digest::getcount (void) const {
    return d_count;
  }

  // get the digest message length
  
  t_long Digest::getlength (void) const {
    return d_length;
  }
   
  // copy some bytes in the buffer and return the number of copied data
  
  long Digest::copy (const t_byte* data, const long size) {
    long result = 0;
    while ((d_count < d_size) && (result < size)) {
      p_data[d_count++] = data[result++];
      d_length++;
    }
    return result;
  }
  

  // copy some bytes in the buffer from a stream
  
  long Digest::copy (Input& is) {
    long result = 0;
    while (is.iseof () == false) {
      p_data[d_count++] = is.read ();
      d_length++;
      result++;
    }
    return result;
  }
};
